@@ -1,7 +1,7 @@
 /*
  * gtk_anyone.c: Show the user tree as a Gtk+ widget
  * David Maze <dmaze@mit.edu>
- * $Id: gtkanyone.c,v 1.25 2002/05/13 22:54:31 dmaze Exp $
+ * $Id: gtkanyone.c,v 1.28 2002/12/17 02:25:16 dmaze Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -143,15 +143,7 @@ static void build_children(GtkAnyone *anyone)
     /* rebuild_children() will create groups for users if they don't
      * already exist, to get around gtk+ brokenness. */
     if (zgroup_groups(group))
-    {
-      subtree = gtk_anyone_new(group);
-      gtk_widget_show(GTK_WIDGET(subtree));
-      gtk_tree_item_set_subtree(GTK_TREE_ITEM(child), subtree);
-      if (zgroup_get_expanded(group))
-        gtk_tree_item_expand(GTK_TREE_ITEM(child));
-      else
-        gtk_tree_item_collapse(GTK_TREE_ITEM(child));
-    }
+      gtk_zgroup_item_force_subtree(GTK_ZGROUP_ITEM(child));
   }
 }
 
@@ -326,22 +318,22 @@ static void rebuild_children(GtkAnyone *anyone)
   {
     /* Every item now should be a group item. */
     GtkZGroupItem *item = GTK_ZGROUP_ITEM(lwidget->data);
-    /* Check it for a child GtkAnyone object. */
-    GtkWidget *subtree = GTK_TREE_ITEM_SUBTREE(GTK_TREE_ITEM(item));
-    if (subtree == NULL)
-    {
-      /* Oops, we lost the subtree (or never had it). */
-      zgroup *group = gtk_zgroup_item_group(item);
-      subtree = gtk_anyone_new(group);
-      gtk_widget_show(subtree);
-      gtk_tree_item_set_subtree(GTK_TREE_ITEM(item), subtree);
-      if (zgroup_get_expanded(group))
-        gtk_tree_item_expand(GTK_TREE_ITEM(item));
-      else
-        gtk_tree_item_collapse(GTK_TREE_ITEM(item));
-    }
+    GtkWidget *subtree = gtk_zgroup_item_force_subtree(item);
     
     rebuild_children(GTK_ANYONE(subtree));
+
+    /* Possible situations here:
+     * (1) subtree has children, and exists, and we want it to exist.
+     * (2) subtree started with children, but doesn't have them anymore.
+     *     In this case, subtree is invalid since Gtk helpfully destroyed
+     *     it for us.
+     * (3) subtree started with no children; we created it, and didn't
+     *     add anything to it, and it now exists and is empty.  In this
+     *     case (only), we want to get rid of it.
+     */
+    if (GTK_TREE_ITEM_SUBTREE(item))
+      if (gtk_container_children(GTK_CONTAINER(subtree)) == NULL)
+        gtk_tree_item_remove_subtree(GTK_TREE_ITEM(item));
   }
 }
 
